from ..Log import *

LogCritical("system incompatible with 'MGE._InputsEmulator'", exit_code=0)

def MousePress(button):
    pass

def MouseRelease(button):
    pass

def MouseClick(button):
    pass

def MouseScroll(scroll):
    pass

def KeyboardPress(key):
    pass

def KeyboardRelease(key):
    pass

def KeyboardClick(key):
    pass
